/*
 * radio.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 * When		Who		What
 * 20050319	F.PECOURT	modified functions calls for new Audio IDs and
 *				radiotransfer for vdr 1.3.22. First dirty hacks
 *				still one problem: blank image when switching
 *				from one radio to an other. Clear missing!
 */

#include <vdr/plugin.h>
#include <vdr/status.h>
#include <vdr/config.h>
#include <vdr/interface.h>
#include "getopt.h"
#include "radiotransfer.h"
//file exists 
#include <sys/stat.h>

static const char *VERSION        = "0.0.6";
static const char *DESCRIPTION    = "Radio Background Image plugin";
static const char *MAINMENUENTRY  = NULL;

int Activate=false;

class cPluginRadio : public cPlugin, cStatus {
private:
  // Add any member variables or functions you may need here.
  cRadioTransfer *_radioTransfer;
  const char *ConfigDir;
  bool ConfigDirParam;
  int newActivate;
public:
  cPluginRadio(void);
  virtual ~cPluginRadio();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
protected:
    virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber);
    bool file_exists(const char * filename);
};

cPluginRadio::cPluginRadio(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PROADUCE ANY OUTPUT!
  _radioTransfer = NULL;
  ConfigDirParam = false;
}

cPluginRadio::~cPluginRadio()
{
  // Clean up after yourself!
  if (ConfigDir) delete(ConfigDir);
}

const char *cPluginRadio::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return   "  -f dir,   --files=dir      use dir as image directory (default: <vdrconfig>/plugins/radio)\n";
}

// --- cMenuSetupRadio -------------------------------------------------------

class cMenuSetupRadio : public cMenuSetupPage {
private:
  int newActivate;
protected:
  virtual void Store(void);
public:
  cMenuSetupRadio(void);
  };

cMenuSetupRadio::cMenuSetupRadio(void)
{
//  newGreetingTime = GreetingTime;
//  newUseAlternateGreeting = UseAlternateGreeting;
//  Add(new cMenuEditIntItem( tr("Greeting time (s)"),      &newGreetingTime));
//  Add(new cMenuEditBoolItem(tr("Use alternate greeting"), &newUseAlternateGreeting));
  newActivate = Activate;
  Add(new cMenuEditBoolItem( tr("Activate "),      &newActivate));
  }

void cMenuSetupRadio::Store(void)
{
  SetupStore("Activate", Activate = newActivate);
}


bool cPluginRadio::ProcessArgs(int argc, char *argv[])
{
	// Implement command line argument processing here if applicable.
	static struct option long_options[] = {
		{ "files",      required_argument, NULL, 'f' },
		{ NULL }
	};
    
	int c;
	while ((c = getopt_long(argc, argv, "f:", long_options, NULL)) != -1) {
		switch (c) {
			case 'f':
				fprintf(stderr, "arg: %s\n", optarg);
				ConfigDir = strdup(optarg);
				ConfigDirParam = true;
				break;
			default:
				fprintf(stderr, "arg char: %c\n", c);
				return false;
			}
	}
	return true;
}

bool cPluginRadio::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  return true;
}

bool cPluginRadio::Start(void)
{
  // Start any background activities the plugin shall perform.
   fprintf(stderr,"vdr: RADIO PLUGIN START\n");
    if (!ConfigDirParam) ConfigDir = strdup(ConfigDirectory( Name() ));
  return true;
}

void cPluginRadio::Stop(void)
{
}

void cPluginRadio::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}   

cOsdObject *cPluginRadio::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  return NULL;
}

cMenuSetupPage *cPluginRadio::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return new cMenuSetupRadio;

//  return NULL;
}

bool cPluginRadio::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  if      (!strcasecmp(Name, "Activate"))         Activate = atoi(Value);
  else
     return false;
  return true;
}

void cPluginRadio::ChannelSwitch(const cDevice *Device, int ChannelNumber)
{
        const cChannel *chan;
	char *image;
  	int apid[2] = {0, 0};
	if (Activate==false) return ;
        if ( cDevice::CurrentChannel() == ChannelNumber ){
	
		chan = ChannelNumber ? Channels.GetByNumber(ChannelNumber) : NULL;

		if (chan != NULL && chan->Vpid() == 0){
			asprintf(&image, "%s/%s.mpg", ConfigDir, chan->Name());
			if ( !file_exists( image ) ){
				fprintf(stderr, "vdr: radio background image not found: %s (Channelname: %s)\n", image, chan->Name());
				dsyslog("vdr: radio background image not found: %s (Channelname: %s)\n", image, chan->Name());
				free(image);
				asprintf(&image, "%s/radio.mpg", ConfigDir);
				if ( !file_exists( image ) ){
					dsyslog("vdr: radio background image not found: %s (Channelname: %s)\n", image, chan->Name());
					fprintf(stderr, "vdr: radio background image not found: %s (Channelname: %s)\n", image, chan->Name());
				}
			}
			apid[0]=chan->Apid(0);
			fprintf(stderr, "*PLUGIN RADIO-IMAGE apid = %d   Channelname: %s Use image : %s\n", chan->Apid(0), chan->Name(),image);
			cControl::Launch(new cRadioTransferControl(cDevice::ActualDevice(), 0, apid , NULL, chan->Ca(), image));
			free(image);
		}
	}
}

bool cPluginRadio::file_exists(const char * filename) {
        struct stat file_stat;
        return ((stat(filename, &file_stat) == 0) ? true:false);
}
//cRadioTransferControl::cRadioTransferControl(cDevice *ReceiverDevice, int VPid, int APid1, int APid2, int DPid1, int DPid2, int CA, const char *Image)
//cRadioTransferControl::cRadioTransferControl(cDevice *ReceiverDevice, int VPid, int Apid[], int Dpid[], int CA, const char *Image)

VDRPLUGINCREATOR(cPluginRadio); // Don't touch this!
