/*
 * radiotransfer.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "radiotransfer.h"

#define TRANSFERBUFSIZE  MEGABYTE(2)
#define POLLTIMEOUTS_BEFORE_DEVICECLEAR 3

// --- cRadioTransfer -------------------------------------------------------------

//cRadioTransfer::cRadioTransfer(int VPid, int APid1, int APid2, int DPid1, int DPid2, int CA, const char *Image)
//:cReceiver(CA, -1, 5, VPid, APid1, APid2, DPid1, DPid2)

cRadioTransfer::cRadioTransfer(int VPid, int APid[], int DPid[], int CA, const char *Image)
:cReceiver(CA, -1, VPid, APid, DPid, NULL)

#if VDRVERSNUM > 10300	
,cThread("radiotransfer")
#else
,cThread()
#endif
{
  dsyslog("new RadioTransfer");
#if VDRVERSNUM > 10310
  ringBuffer = new cRingBufferLinear(TRANSFERBUFSIZE, TS_SIZE * 2, true, "RadioTransfer");
#else
   ringBuffer = new cRingBufferLinear(TRANSFERBUFSIZE, TS_SIZE * 2, true);
#endif
  remux = new cRemux(VPid, APid, DPid, NULL); //VPid, APid1, APid2, DPid1, DPid2
  canToggleAudioTrack = false;
  audioTrack = 0xC0;
  active = false;
  imagepath = Image;
}

cRadioTransfer::~cRadioTransfer()
{
  dsyslog("delete RadioTransfer");
  cReceiver::Detach();
  cPlayer::Detach();
  delete remux;
  delete ringBuffer;
}

void cRadioTransfer::Activate(bool On)
{
  if (On) {
     if (!active)
        Start();
     }
  else if (active) {
     active = false;
     Cancel(3);
     }
}

void cRadioTransfer::Receive(uchar *Data, int Length)
{
  if (IsAttached() && active) {
     int p = ringBuffer->Put(Data, Length);
     if (p != Length && active)
#if VDRVERSNUM > 10300	     
        ringBuffer->ReportOverflow(Length - p);
#else
      esyslog("ERROR: ring buffer overflow (%d bytes dropped)", Length);
#endif
      return;
     }
}

void cRadioTransfer::Action(void)
{
  int PollTimeouts = 0;
  active = true;
  ShowImage(imagepath);
#if VDRVERSNUM > 10318
  uchar *p = NULL;
  int Result = 0;
  while (active) {
  	int Count;
        uchar *b = ringBuffer->Get(Count);
        if (b) {
           if (ringBuffer->Available() > TRANSFERBUFSIZE * 9 / 10) {
              // If the buffer runs full, we have no chance of ever catching up
              // since the data comes in at the same rate as it goes out (it's "live").
              // So let's clear the buffer instead of suffering from permanent
              // overflows.
              dsyslog("clearing transfer buffer to avoid overflows");
              DeviceClear();
              ringBuffer->Clear();
              remux->Clear();
              PlayPes(NULL, 0);
              p = NULL;
#ifdef FW_NEEDS_BUFFER_RESERVE_FOR_AC3
              GotBufferReserve = false;
#endif
              continue;
              }
           Count = remux->Put(b, Count);
           if (Count)
              ringBuffer->Del(Count);
           }
        if (!p)
           p = remux->Get(Result);
        if (p) {
           cPoller Poller;
           if (DevicePoll(Poller, 100)) {
              PollTimeouts = 0;
              int w = PlayPes(p, Result);
              if (w > 0) {
                 p += w;
                 Result -= w;
                 remux->Del(w);
                 if (Result <= 0)
                    p = NULL;
                 }
              else if (w < 0 && FATALERRNO)
                 LOG_ERROR;
              }
           else {
              PollTimeouts++;
              if (PollTimeouts == POLLTIMEOUTS_BEFORE_DEVICECLEAR) {
                 dsyslog("clearing device because of consecutive poll timeouts");
                 DeviceClear();
                 ringBuffer->Clear();
                 remux->Clear();
                 PlayPes(NULL, 0);
                 p = NULL;
#ifdef FW_NEEDS_BUFFER_RESERVE_FOR_AC3
                 GotBufferReserve = false;
#endif
                 }
              }
           }
        }
#elif VDRVERSNUM > 10310
  uchar *p = NULL;
  int Result = 0;
  while (active) {
        int Count;
        uchar *b = ringBuffer->Get(Count);
        if (b) {
           if (Count > TRANSFERBUFSIZE * 2 / 3) {
              // If the buffer runs full, we have no chance of ever catching up
              // since the data comes in at the same rate as it goes out (it's "live").
              // So let's clear the buffer instead of suffering from permanent
              // overflows.
              dsyslog("clearing transfer buffer to avoid overflows");
		fprintf(stderr,"clearing transfer buffer to avoid overflows");
              ringBuffer->Clear();
              remux->Clear();
              p = NULL;
              continue;
              }
           Count = remux->Put(b, Count);
           if (Count)
              ringBuffer->Del(Count);
           }
        if (!p && (p = remux->Get(Result)) != NULL){
           StripAudioPackets(p, Result, audioTrack);
	}
        if (p) {
           cPoller Poller;
           if (DevicePoll(Poller, 100)) {
              PollTimeouts = 0;
              int w = PlayPes(p, Result);
              if (w > 0) {
                 p += w;
                 Result -= w;
                 remux->Del(w);
                 if (Result <= 0)
                    p = NULL;
                 }
              else if (w < 0 && FATALERRNO)
                 LOG_ERROR;
              }
           else {
              PollTimeouts++;
              if (PollTimeouts == POLLTIMEOUTS_BEFORE_DEVICECLEAR) {
                 dsyslog("clearing device because of consecutive poll timeouts");
		 fprintf(stderr,"clearing device because of consecutive poll timeouts");
		 
                 DeviceClear();
                 ringBuffer->Clear();
                 remux->Clear();
                 p = NULL;
		 ShowImage(imagepath);
                 }
              }
           }
        }
#else
	while (active) {
		int r;
		const uchar *b = ringBuffer->Get(r);
		
		if (b) {
			int Count = r, Result;
			uchar *p = remux->Process(b, Count, Result);
			ringBuffer->Del(Count);
			if (p) {
				StripAudioPackets(p, Result, audioTrack);
				while (Result > 0 && active) {
					cPoller Poller;
					if (DevicePoll(Poller, 100)) {
						PollTimeouts = 0;
						int w = PlayPes(p, Result);
						if (w > 0) {
							p += w;
							Result -= w;
						}
						else if (w < 0 && FATALERRNO) {
							LOG_ERROR;
							break;
						}
					}
					else {
						PollTimeouts++;
						if (PollTimeouts == POLLTIMEOUTS_BEFORE_DEVICECLEAR) {
							dsyslog("clearing device because of consecutive poll timeouts");
							DeviceClear();
						}
					}
				}
			}
		}
		else
			usleep(1); // this keeps the CPU load low
		
	}
#endif
  active = false;
}

void cRadioTransfer::StripAudioPackets(uchar *b, int Length, uchar Except)
{
  for (int i = 0; i < Length - 6; i++) {
      if (b[i] == 0x00 && b[i + 1] == 0x00 && b[i + 2] == 0x01) {
         uchar c = b[i + 3];
         int l = b[i + 4] * 256 + b[i + 5] + 6;
         switch (c) {
           case 0xBD: // dolby
                if (Except)
                   PlayPes(&b[i], l);
                // continue with deleting the data - otherwise it disturbs DVB replay
           case 0xC0 ... 0xC1: // audio
                if (c == 0xC1)
                   canToggleAudioTrack = true;
                if (!Except || c != Except)
                   memset(&b[i], 0x00, min(l, Length-i));
                break;
           case 0xE0 ... 0xEF: // video
		memset(&b[i], 0x00, min(l, Length-i));
                break;
           default:
                //esyslog("ERROR: unexpected packet id %02X", c);
                l = 0;
           }
	 PlayPes(&b[i], l);
         if (l)
            i += l - 1; // the loop increments, too!
         }
      /*XXX
      else
         esyslog("ERROR: broken packet header");
         XXX*/
      }
}

int cRadioTransfer::NumAudioTracks(void) const
{
  return canToggleAudioTrack ? 2 : 1;
}

const char **cRadioTransfer::GetAudioTracks(int *CurrentTrack) const
{
  if (NumAudioTracks()) {
     if (CurrentTrack)
        *CurrentTrack = (audioTrack == 0xC0) ? 0 : 1;
     static const char *audioTracks1[] = { "Audio 1", NULL };
     static const char *audioTracks2[] = { "Audio 1", "Audio 2", NULL };
     return NumAudioTracks() > 1 ? audioTracks2 : audioTracks1;
     }
  return NULL;
}

void cRadioTransfer::SetAudioTrack(eTrackType Type, const tTrackId *TrackId)
{
  if ((audioTrack == 0xC0) != (Type == 0)) {
     audioTrack = (Type == 1) ? 0xC1 : 0xC0;
     DeviceClear();
     }
}

void cRadioTransfer::ShowImage (const char *file)
{
	uchar *buffer;
	int fd;
	struct stat st;
	struct video_still_picture sp;
	if ((fd = open (file, O_RDONLY)) >= 0)
	{
		fstat (fd, &st);
		sp.iFrame = (char *) malloc (st.st_size);
		if (sp.iFrame)
		{
			sp.size = st.st_size;
			if (read (fd, sp.iFrame, sp.size) > 0)
			{
				buffer = (uchar *) sp.iFrame;  
#ifdef USE_STILLPICTURE
				DeviceStillPicture(buffer, sp.size);
#else
				for (int i = 1; i <= 25; i++)
				{
					send_pes_packet (buffer, sp.size, i);
				}
#endif
			}
			free (sp.iFrame);
		}
		else
		{
		}
		close (fd);
	}
	else
	{
	}
}

void cRadioTransfer::send_pes_packet(unsigned char *data, int len, int timestamp)
{
#define PES_MAX_SIZE 2048
	int ptslen = timestamp ? 5 : 1;
	static unsigned char pes_header[PES_MAX_SIZE];
	pes_header[0] = pes_header[1] = 0;
	pes_header[2] = 1;
	pes_header[3] = 0xe0;
	while(len > 0)
	{
		int payload_size = len;
		if(6 + ptslen + payload_size > PES_MAX_SIZE)
			payload_size = PES_MAX_SIZE - (6 + ptslen);
		pes_header[4] = (ptslen + payload_size) >> 8;
		pes_header[5] = (ptslen + payload_size) & 255;
		if(ptslen == 5)
		{
			int x;
			x = (0x02 << 4) | (((timestamp >> 30) & 0x07) << 1) | 1;
			pes_header[8] = x;
			x = ((((timestamp >> 15) & 0x7fff) << 1) | 1);
			pes_header[7] = x >> 8;
			pes_header[8] = x & 255;
			x = ((((timestamp) & 0x7fff) < 1) | 1);
			pes_header[9] = x >> 8;
			pes_header[10] = x & 255;
		} else
		{
			pes_header[6] = 0x0f;
		}
		memcpy(&pes_header[6 + ptslen], data, payload_size);
		PlayPes(pes_header, 6 + ptslen + payload_size);
		len -= payload_size;
		data += payload_size;
		ptslen = 1;
	}
}


// --- cRadioTransferControl ------------------------------------------------------

cDevice *cRadioTransferControl::receiverDevice = NULL;

//cRadioTransferControl::cRadioTransferControl(cDevice *ReceiverDevice, int VPid, int APid1, int APid2, int DPid1, int DPid2, int CA, const char *Image)
//:cControl(transfer = new cRadioTransfer(VPid, APid1, APid2, DPid1, DPid2, CA, Image), true)


cRadioTransferControl::cRadioTransferControl(cDevice *ReceiverDevice, int VPid, int APid[], int DPid[], int CA, const char *Image)
:cControl(transfer = new cRadioTransfer(VPid, APid, DPid, CA, Image), true)
{
  cDevice::PrimaryDevice()->AttachPlayer(transfer );
  ReceiverDevice->AttachReceiver(transfer);
  receiverDevice = ReceiverDevice;
}

cRadioTransferControl::~cRadioTransferControl()
{
  receiverDevice = NULL;
  delete transfer;
}
